/* DO NOT EDIT. This is auto-generated from feature.sh */
#define CK_F_PR_ADD_16
#define CK_F_PR_ADD_32
#define CK_F_PR_ADD_64
#define CK_F_PR_ADD_8
#define CK_F_PR_ADD_CHAR
#define CK_F_PR_ADD_INT
#define CK_F_PR_ADD_PTR
#define CK_F_PR_ADD_UINT
#define CK_F_PR_AND_16
#define CK_F_PR_AND_32
#define CK_F_PR_AND_64
#define CK_F_PR_AND_8
#define CK_F_PR_AND_CHAR
#define CK_F_PR_AND_INT
#define CK_F_PR_AND_PTR
#define CK_F_PR_AND_UINT
#define CK_F_PR_BTC_16
#define CK_F_PR_BTC_32
#define CK_F_PR_BTC_64
#define CK_F_PR_BTC_INT
#define CK_F_PR_BTC_PTR
#define CK_F_PR_BTC_UINT
#define CK_F_PR_BTR_16
#define CK_F_PR_BTR_32
#define CK_F_PR_BTR_64
#define CK_F_PR_BTR_INT
#define CK_F_PR_BTR_PTR
#define CK_F_PR_BTR_UINT
#define CK_F_PR_BTS_16
#define CK_F_PR_BTS_32
#define CK_F_PR_BTS_64
#define CK_F_PR_BTS_INT
#define CK_F_PR_BTS_PTR
#define CK_F_PR_BTS_UINT
#define CK_F_PR_CAS_16
#define CK_F_PR_CAS_16_8
#define CK_F_PR_CAS_16_8_VALUE
#define CK_F_PR_CAS_16_VALUE
#define CK_F_PR_CAS_32
#define CK_F_PR_CAS_32_4
#define CK_F_PR_CAS_32_4_VALUE
#define CK_F_PR_CAS_32_VALUE
#define CK_F_PR_CAS_64
#define CK_F_PR_CAS_64_2
#define CK_F_PR_CAS_64_2_VALUE
#define CK_F_PR_CAS_64_VALUE
#define CK_F_PR_CAS_8
#define CK_F_PR_CAS_8_16
#define CK_F_PR_CAS_8_16_VALUE
#define CK_F_PR_CAS_8_VALUE
#define CK_F_PR_CAS_CHAR
#define CK_F_PR_CAS_CHAR_16
#define CK_F_PR_CAS_CHAR_16_VALUE
#define CK_F_PR_CAS_CHAR_VALUE
#define CK_F_PR_CAS_INT
#define CK_F_PR_CAS_INT_4
#define CK_F_PR_CAS_INT_4_VALUE
#define CK_F_PR_CAS_INT_VALUE
#define CK_F_PR_CAS_PTR
#define CK_F_PR_CAS_PTR_2
#define CK_F_PR_CAS_PTR_2_VALUE
#define CK_F_PR_CAS_PTR_VALUE
#define CK_F_PR_CAS_DOUBLE
#define CK_F_PR_CAS_DOUBLE_2
#define CK_F_PR_CAS_DOUBLE_VALUE
#define CK_F_PR_CAS_UINT
#define CK_F_PR_CAS_UINT_4
#define CK_F_PR_CAS_UINT_4_VALUE
#define CK_F_PR_CAS_UINT_VALUE
#define CK_F_PR_DEC_16
#define CK_F_PR_DEC_16_ZERO
#define CK_F_PR_DEC_32
#define CK_F_PR_DEC_32_ZERO
#define CK_F_PR_DEC_64
#define CK_F_PR_DEC_64_ZERO
#define CK_F_PR_DEC_8
#define CK_F_PR_DEC_8_ZERO
#define CK_F_PR_DEC_CHAR
#define CK_F_PR_DEC_CHAR_ZERO
#define CK_F_PR_DEC_INT
#define CK_F_PR_DEC_INT_ZERO
#define CK_F_PR_DEC_PTR
#define CK_F_PR_DEC_PTR_ZERO
#define CK_F_PR_DEC_UINT
#define CK_F_PR_DEC_UINT_ZERO
#define CK_F_PR_FAA_16
#define CK_F_PR_FAA_32
#define CK_F_PR_FAA_64
#define CK_F_PR_FAA_8
#define CK_F_PR_FAA_CHAR
#define CK_F_PR_FAA_INT
#define CK_F_PR_FAA_PTR
#define CK_F_PR_FAA_UINT
#define CK_F_PR_FAS_16
#define CK_F_PR_FAS_32
#define CK_F_PR_FAS_64
#define CK_F_PR_FAS_8
#define CK_F_PR_FAS_CHAR
#define CK_F_PR_FAS_INT
#define CK_F_PR_FAS_PTR
#define CK_F_PR_FAS_UINT
#define CK_F_PR_FAS_DOUBLE
#define CK_F_PR_FENCE_LOAD
#define CK_F_PR_FENCE_LOAD_DEPENDS
#define CK_F_PR_FENCE_MEMORY
#define CK_F_PR_FENCE_STORE
#define CK_F_PR_FENCE_STRICT_LOAD
#define CK_F_PR_FENCE_STRICT_LOAD_DEPENDS
#define CK_F_PR_FENCE_STRICT_MEMORY
#define CK_F_PR_FENCE_STRICT_STORE
#define CK_F_PR_INC_16
#define CK_F_PR_INC_16_ZERO
#define CK_F_PR_INC_32
#define CK_F_PR_INC_32_ZERO
#define CK_F_PR_INC_64
#define CK_F_PR_INC_64_ZERO
#define CK_F_PR_INC_8
#define CK_F_PR_INC_8_ZERO
#define CK_F_PR_INC_CHAR
#define CK_F_PR_INC_CHAR_ZERO
#define CK_F_PR_INC_INT
#define CK_F_PR_INC_INT_ZERO
#define CK_F_PR_INC_PTR
#define CK_F_PR_INC_PTR_ZERO
#define CK_F_PR_INC_UINT
#define CK_F_PR_INC_UINT_ZERO
#define CK_F_PR_LOAD_16
#define CK_F_PR_LOAD_16_8
#define CK_F_PR_LOAD_32
#define CK_F_PR_LOAD_32_4
#define CK_F_PR_LOAD_64
#define CK_F_PR_LOAD_64_2
#define CK_F_PR_LOAD_8
#define CK_F_PR_LOAD_8_16
#define CK_F_PR_LOAD_CHAR
#define CK_F_PR_LOAD_CHAR_16
#define CK_F_PR_LOAD_INT
#define CK_F_PR_LOAD_INT_4
#define CK_F_PR_LOAD_PTR
#define CK_F_PR_LOAD_PTR_2
#define CK_F_PR_LOAD_DOUBLE
#define CK_F_PR_LOAD_UINT
#define CK_F_PR_LOAD_UINT_4
#define CK_F_PR_NEG_16
#define CK_F_PR_NEG_16_ZERO
#define CK_F_PR_NEG_32
#define CK_F_PR_NEG_32_ZERO
#define CK_F_PR_NEG_64
#define CK_F_PR_NEG_64_ZERO
#define CK_F_PR_NEG_8
#define CK_F_PR_NEG_8_ZERO
#define CK_F_PR_NEG_CHAR
#define CK_F_PR_NEG_CHAR_ZERO
#define CK_F_PR_NEG_INT
#define CK_F_PR_NEG_INT_ZERO
#define CK_F_PR_NEG_PTR
#define CK_F_PR_NEG_PTR_ZERO
#define CK_F_PR_NEG_UINT
#define CK_F_PR_NEG_UINT_ZERO
#define CK_F_PR_NOT_16
#define CK_F_PR_NOT_32
#define CK_F_PR_NOT_64
#define CK_F_PR_NOT_8
#define CK_F_PR_NOT_CHAR
#define CK_F_PR_NOT_INT
#define CK_F_PR_NOT_PTR
#define CK_F_PR_NOT_UINT
#define CK_F_PR_OR_16
#define CK_F_PR_OR_32
#define CK_F_PR_OR_64
#define CK_F_PR_OR_8
#define CK_F_PR_OR_CHAR
#define CK_F_PR_OR_INT
#define CK_F_PR_OR_PTR
#define CK_F_PR_OR_UINT
#define CK_F_PR_STORE_16
#define CK_F_PR_STORE_32
#define CK_F_PR_STORE_64
#define CK_F_PR_STORE_8
#define CK_F_PR_STORE_CHAR
#define CK_F_PR_STORE_INT
#define CK_F_PR_STORE_DOUBLE
#define CK_F_PR_STORE_PTR
#define CK_F_PR_STORE_UINT
#define CK_F_PR_SUB_16
#define CK_F_PR_SUB_32
#define CK_F_PR_SUB_64
#define CK_F_PR_SUB_8
#define CK_F_PR_SUB_CHAR
#define CK_F_PR_SUB_INT
#define CK_F_PR_SUB_PTR
#define CK_F_PR_SUB_UINT
#define CK_F_PR_XOR_16
#define CK_F_PR_XOR_32
#define CK_F_PR_XOR_64
#define CK_F_PR_XOR_8
#define CK_F_PR_XOR_CHAR
#define CK_F_PR_XOR_INT
#define CK_F_PR_XOR_PTR
#define CK_F_PR_XOR_UINT

